/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/





#include "DataStructures.h"
#include "version1_files/RootData.h"
#include "version1_files/RootflyData.h"
#include "version1_files/RootOverTime.h"
#include "version1_files/Session_Array.h"
#include "TestDataStructure.h"
#include "ImageFileFormat.h"
#include "ImageToWindowConverter.h"

#include <math.h>
#include <algorithm>
#include <sstream>
#include <string>

extern DebugLogFile g_debug_logfile;

//////////////////////////
//Class RPoint functions// 
//////////////////////////

RPoint::RPoint()
{
  
}

RPoint::RPoint(int _x, int _y, int _window )
{
  x = _x;
  y = _y;
  window = _window;
}

void RPoint::Copy(RPoint &point2)
{
  point2.x = x;
  point2.y = y;
  point2.window = window;
}

/////////////////////////////
//Class RDiameter functions// 
/////////////////////////////


RDiameter::RDiameter()
{
  
}

RDiameter::RDiameter( double rad, RPoint &rp )
{
  radius = rad;
  r_point = rp;
}

RDiameter::RDiameter( double rad, const RPoint& rp )
{
  radius = rad;
  r_point = rp;
}

void RDiameter::Copy(RDiameter &diam2)
{
  diam2.radius = radius;
  r_point.Copy(diam2.r_point);
}

////////////////////////////////
//Class RootInstance functions//
////////////////////////////////

/**
* Default constructor for new Sessions
*/
RootInstance::RootInstance()
{
  m_color = COLOR_WHITE;
//  m_condition_code;
//  m_session_number = -1;
  m_is_alive = true;                  
}

/**
* Default destructor
*/
RootInstance::~RootInstance()
{

}

void RootInstance::Copy(RootInstance &inst2)
{
  inst2.SetColor(m_color);
  inst2.SetConditionCode(m_condition_code);
  inst2.SetSessionNumber(m_session_number);
  if(m_is_alive) inst2.SetAlive();
  else inst2.SetDead();
  for( int i = 0; i < m_points.size(); i++ )
  {
    RPoint *point = inst2.CreatePoint();
    m_points.at(i).Copy(*point);
  }
  for( int j = 0; j < m_diameters.size(); j++ )
  {
    RDiameter *diam = inst2.CreateDiameter();
    m_diameters.at(j).Copy(*diam);
  }
}

RPoint* RootInstance::CreatePoint( )
{  
  RPoint pnt;
  m_points.push_back(pnt);
  return &m_points.back();
}

RDiameter* RootInstance::CreateDiameter()
{
  RDiameter dia;
  m_diameters.push_back(dia);
  return &m_diameters.back();
}


/**
*
* Clears the vectors of points and diameters
* keeps session#,color,condition, and live
* @returns: true on success, false on fail.
*/
bool RootInstance::ClearAll()
{
  m_points.clear();
  m_diameters.clear();
  
  Color m_color = COLOR_WHITE;
  m_condition_code = "";
  m_session_number = -1;
  m_is_alive = false;
  
  return m_points.size() == 0 && m_diameters.size() == 0;
}


/**
@return: 0 if erase successful, 1 if failure
*/
bool RootInstance::DeletePoint(int index)
{
  if(index<0||index>m_points.size()-1||m_points.size()<3)
    return false;
  
  RPoint *returned_point = m_points.erase(m_points.begin() + index);
  return returned_point == NULL?true:false;     
}

/**
@return: true if move successful, false if failure
*/
bool RootInstance::MovePoint(int index, int x, int y)
{
  if(index<0||index>m_points.size()-1)
    return false;

  (m_points.begin() + index)->x = x;
  (m_points.begin() + index)->y = y;

  return true;
}

/**
@return: true if move successful, false if failure
*/
bool RootInstance::ChangeWindowOfPoint(int index, int w)
{
  if(index<0||index>m_points.size()-1)
    return false;

  (m_points.begin() + index)->window = w; 

  return true;
} 

/**
@return: 0 if erase successful, 1 if failure
*/
bool RootInstance::DeleteDiameter(int index)
{
  if(index<0||index>m_diameters.size()-1)
    return false;
  
  RDiameter *returned_diameter = m_diameters.erase(m_diameters.begin() + index);
  return returned_diameter == NULL?true:false;
}

/**
@return: 0 if successful, 1 if failure
*/
bool RootInstance::MoveDiameter(int index, int x, int y)
{
  
  if (index < 0 || index > m_diameters.size()-1)
    return false;
  
  m_diameters[ index ].r_point.x = x;
  m_diameters[ index ].r_point.y = y;

  return true;
}  

/**
@return: true if move successful, false if failure
*/
bool RootInstance::ChangeWindowOfDiameter(int index, int w)
{
  if(index<0||index>m_diameters.size()-1)
    return false;

  m_diameters [ index ].r_point.window = w;

  return true;
} 

/**
@return: 0 if successful, 1 if failure
*/
bool RootInstance::ResizeDiameter(int index, double radius)
{
  
  if (index < 0 || index > m_diameters.size()-1)
    return false;
  
  m_diameters[ index ].radius = radius;

  return true;
}  

/**
* This function will fill a given vector with numbers representing the windows this root instance exists within.
* @param window_list pointer to a vector of ints to be populated with window numbers
* @return Nothing
*/
void RootInstance::GetWindowNumbers(vector<int>* window_list) const
{
  int num_points = m_points.size(), i=0, j=0;
  bool window_recorded = false;

  window_list->clear();

  //Loop through all points in this root instance.
  for(i=0; i<num_points; i++) 
  {
    //Checks to see if the window for a point is already in the list.
    for(j=0; j<window_list->size(); j++) 
    {
      if(m_points.at(i).window == window_list->at(j)) 
      {
          window_recorded=true;
      }
    }

    //If the window of the point was not in the list, add it.
    if(!window_recorded)
    {
      window_list->push_back(m_points.at(i).window);
    }

    //Reset the window setting for the next loop iteration.
    window_recorded = false;
  }

  return;
}

void RootInstance::SetColor(int color)
{
  switch(color)
  {
  case 0:
    m_color = COLOR_WHITE;
    break;
  case 1:
    m_color = COLOR_LIGHTBROWN;
    break;
  case 2:
    m_color = COLOR_DARKBROWN;
    break;
  case 3:
    m_color = COLOR_BLACK;
    break;
  default:
    m_color = COLOR_WHITE;
  }
}

/**
* Returns the length of the current root in a given window
* @param window Window to find length information within
* @param conversion The pixel-to-millimeter conversion ratio from the application.
* @return double which represents length between points in the given window.  -1 indicates that failure occurred, possibly because no points were located in the given window.
*/
const double RootInstance::FindLengthInWindow(int window, float conversion) const
{
  int i,num_points;
  double length = 0.;
  vector<RPoint> points_in_window;

  //Error out if the window number is invalid
  if(window < 0) 
  {
    return -1;
  }

  num_points = m_points.size();

  //If a point is in the window, add it to a vector for later processing
  for(i=0; i<num_points; i++) 
  {
    if(m_points.at(i).window == window) 
    {
      points_in_window.push_back(m_points.at(i));
    }
  }
  
  //Error out if no points were found.
  if(points_in_window.empty()) 
  {
    return -1;
  }

  //Compute the distance between all points in the given window.
  for(i=0; i<points_in_window.size()-1; i++) 
  {
    length += sqrt(pow(points_in_window.at(i).x - points_in_window.at(i+1).x, 2) + pow(points_in_window.at(i).y - points_in_window.at(i+1).y,2));
  }

  return length/conversion;
}

/**
* Returns the average diameter, currently there is only one diameter.  This function is ready for multiple diameters.
* @param conversion The pixel-to-millimeter conversion ratio from the application.
* @return double representing (total diameter size)/(number of diameters) / conversion, and -1 if there is an error.
*/
const double RootInstance::GetAverageDiameter(float conversion) const
{
  int i=0;
  double diameter=0;

  assert(m_diameters.size() > 0);

  //This should never happen.  If it does, there is a bug.
  if(m_diameters.size() == 0)
  {
    return -1;
  }

  //Get a sum of the diameters, used to create a simple average.
  for(i=0; i<m_diameters.size(); i++) {
    diameter += m_diameters.at(i).radius;
  }

  return (diameter/m_diameters.size())/conversion;
}

////////////////////////
//Class Root Functions//
////////////////////////

/**
*
* Constructor
*/
Root::Root( int ID )
{
  m_id = ID;
  m_window_id = 0;
  //AfxMessageBox("New Root Created");
}

Root::Root()
{
  
}

/**
* Deconstructor
*/
Root::~Root()
{
}

void Root::Copy(Root &root2)
{
  root2.SetID(m_id);
  root2.SetPrimaryWindow(m_primary_window);
  root2.SetWindowID(m_window_id);
  for( int i = 0; i < m_instances.size(); i++ )
  {
    RootInstance *inst = root2.CreateRootInstance();
    m_instances.at(i).Copy(*inst);
  }
}


/**
* Creates a new session for the root and returns a reference to it.
*/
RootInstance* Root::CreateRootInstance(int session_number)
{
  
  // checks to see if root instance with same session number already exists
  // if it does, it returns existing instance
  for (size_t ix=0 ; ix < m_instances.size(); ++ix)
  {
    if (m_instances[ix].GetSessionNumber() == session_number)
    {
      return &m_instances.at(ix);
    }
  }
  
  // TODO: not sure how to sort after adding instance and return it, so currently sorting before add
  std::sort(m_instances.begin(),m_instances.end());
  
  RootInstance new_root_instance;
  new_root_instance.SetSessionNumber(session_number);
//  new_root_instance.SetColor(RootInstance::COLOR_WHITE);
  new_root_instance.SetConditionCode("NO_CODE");

  m_instances.push_back(new_root_instance);
  return &m_instances.back();
}

RootInstance* Root::CreateRootInstance()
{
  
  // TODO: not sure how to sort after adding instance and return it, so currently sorting before add
  std::sort(m_instances.begin(),m_instances.end());
  
  RootInstance new_root_instance;  
  new_root_instance.SetColor(RootInstance::COLOR_DARKBROWN);
  new_root_instance.SetConditionCode("NO_CODE");

  m_instances.push_back(new_root_instance);
  return &m_instances.back();
}

/**
* Removes a root session at position session_number in the m_session vector.
* @param session_number
*/
bool Root::DeleteRootInstance(int session_number)
{
  for (size_t ix=0 ; ix < m_instances.size(); ++ix)
  {
    if (m_instances[ix].GetSessionNumber() == session_number)
    {
      std::vector<RootInstance>::iterator iter = m_instances.begin();
      advance(iter,ix);
      
      RootInstance *returned_instance = m_instances.erase(iter);
      return returned_instance == NULL?true:false;
    }
  }
  return NULL;
}


/**
* Copies a root instance to a new session number.
* @param from int representing the source session number to copy from
* @param to int representing the destination session number that will be set in the root instance object.
* @param notified pointer to boolean which keeps track of whether the user has already been notified they will be overwriting roots
* @param previous_choice pointer to an integer representing the users past choice given the previous overwrite notification (IDOK or IDCANCEL)
* @return bool indicating success(true) or failure(false)
*/
bool Root::CopyRootInstance(int from, int to)
{
  int i=0, j=0, from_index=-1, to_index=-1;
  
  // check input parameters
  assert(from >= 1 && to >= 1);
  if (from < 1 || to < 1 || from == to)  return false;  // invalid session numbers
  if (m_instances.size() == 0)  return false;  // there's nothing in the list, so we can't copy previous

  //This iterator is used in case we need to delete something from the m_instances vector
  vector<RootInstance>::iterator iter = m_instances.begin();

  //Search for the source root instance, and set the new one equal to it.  Also find the destination index for replacement purposes.
  for (i=0 ; i<m_instances.size() ; i++)
  {
    if (m_instances.at(i).GetSessionNumber() == from)  
    { 
      assert(from_index == -1);  // should not be more than one instance in a session
      from_index = i;
    }
    if (m_instances.at(i).GetSessionNumber() == to)
    {
      assert(to_index == -1);  // should not be more than one instance in a session
      to_index = i;
    }
  }

  // No root instance in the source session was found, so return
  if (from_index == -1)  return false;

  //Create a copy of the "previous" root instance
  RootInstance new_root_instance = m_instances.at(from_index);
  new_root_instance.SetSessionNumber(to);

  //If roots already exist in the destination session, ask if the user wants to overwrite them
  if (to_index != -1 )
  {
    //iter += to_index;

    *iter = m_instances.at(to_index);

    //Erase the root if the user selected OK.
    m_instances.erase(iter);
  } 
  else 
  {
    //iter = m_instances.end();
  }

  //This inserts the root into its correct location in the m_instances vector
  //m_instances.insert(iter, 1, new_root_instance);
  m_instances.push_back(new_root_instance);
  
  return true;
}


/**
* Returns a pointer to a RootInstance with the given session_number
* @param session_number
* @return pointer to the root instance for the given session_number
*/
RootInstance* Root::GetRootInstance(int session_number)
{
  for (size_t ix=0 ; ix < m_instances.size(); ++ix)
  {
    if (m_instances[ix].GetSessionNumber() == session_number)
    {
      return &m_instances.at(ix);
    }
  }
  //AfxMessageBox("not found");
  return NULL;
}

/**
* Returns a const pointer to a RootInstance with the given session_number
* @param session_number
* @return const pointer to the root instance for the given session_number
*/
const RootInstance* Root::GetRootInstance(int session_number) const
{
  for (size_t ix=0 ; ix < m_instances.size(); ++ix)
  {
    if (m_instances[ix].GetSessionNumber() == session_number)
    {
      return &m_instances.at(ix);
    }
  }
  //AfxMessageBox(" Root not found");
  return NULL;
}

/**
* Loops through the root instances and returns the highest session number found.
* @return const integer which is the largest session number in each of the root instance objects, -1 if no instances exist.
*/
const int Root::GetLastSessionNumber() const
{
	int i, session=0;

  //Error out if there are no sessions
  if(m_instances.size() == 0) 
  {
    return -1;
  }
	
  //Loop through and return the *highest* session number.  This will work even if the sessions are out of order.
  for(i=0; i<m_instances.size(); i++) 
	{
		if(m_instances.at(i).GetSessionNumber() > session)
			session = m_instances.at(i).GetSessionNumber();
	}

	return session;
}

/**
* Returns the first session for this particular root
* @return const int representing the session number, and -1 if there are no instances of this root.
*/
const int Root::GetFirstSessionNumber() const
{
  int min=10000;//assuming the root does not traverse 10000 sessions
	if(m_instances.size() == 0) 
	{
		return -1;
	}
  
  //Loop through and return the *lowest* session number.  This will work even if the sessions are out of order.
  for(int i=0; i<m_instances.size() ; i++) 
	{
		if(min > m_instances.at(i).GetSessionNumber()) 
		{
			min = m_instances.at(i).GetSessionNumber();
		}
	}

	return min;
}

/**
* Returns the first session this root showed up dead
* @return const int representing the session number this root was first declared dead, -2 if it's not dead and -1 if there are no instances.
*/
const int Root::GetFirstDeadSessionNumber() const
{
	int i,session=10000;  //Assuming this does not traverse more than 10000 sessions.
  bool found = false;

  //Error out if there are no root instances.
  if(m_instances.size() == 0) 
  {
    return -1;
  }

  //Loop through and return the *lowest* dead session number.  This will work even if the sessions are out of order.
	for(i=0; i<m_instances.size(); i++) 
	{
		if(!m_instances.at(i).IsAlive() && session > m_instances.at(i).GetSessionNumber()) 
		{
			session = m_instances.at(i).GetSessionNumber();
		}
	}

  //If the root never died, this is also a special condition, indicated by -2.
  if(session == 10000) 
  {
    return -2;
  }

	return session;
}

/**
* This function sums the length of the root over multiple windows, for each session and returns the largest value.
* @param conversion the conversion ratio to apply to the length.
* @return const double representing the actual maximum length of the root in mm.  -1 if there are no sessions.
*/
const double Root::GetMaxLength(float conversion) const 
{
	vector<int> windows;    //Holds the window numbers where this root exists
	int i=0,j=0;
	double length=0.0, templength;

  //Error out if there are no instances
  if(m_instances.size() == 0) 
  {
    return -1;
  }

  //Loop through all instances, for each one sum the lengths of the root in each window it exists within.  Then
  //  compare each of these "sums" to every other sum finding the largest one for the entire root.
	for(i=0; i<m_instances.size(); i++) 
	{
		templength = 0.0;
		m_instances.at(i).GetWindowNumbers(&windows);

    //Find the sum within each respective window and sum it up.
		for(j=0; j<windows.size(); j++) 
			templength+= m_instances.at(i).FindLengthInWindow(windows.at(j), conversion);
		
		if(templength > length) 
			length = templength;
	}
	
	return length;
	
}

/**
* Searches over all diameters and returns the largest diameter found.
* @param conversion the mm conversion ratio used to convert the pixel lengths to mm.
* @return const double representing the maximum root diameter found, and -1 if there are no root instances.
*/
const double Root::GetMaxDiameter(float conversion) const 
{
	int i=0;
	double diameter=0.0;

  //Error out if there are no root instances
  if(m_instances.size() == 0) 
  {
    return -1;
  }

  //Loop through every instance, taking each average diameter and keep the largest one found
  //  NOTE:  Currently GetAverageDiameter() simply returns the size of one diameter.  The function
  //          is capable of returning an actual average though.
	for(i=0; i<m_instances.size(); i++) 
	{
		if(m_instances.at(i).GetAverageDiameter(conversion) > diameter) 
			diameter = m_instances.at(i).GetAverageDiameter(conversion);
	}
	
	return diameter;
}


/**
*
* Clears the vector of instances, m_instances
* 
* @returns: true on success, false on fail.
*/
bool Root::ClearAll()
{
  m_instances.clear();
  
  m_id = -1;
  m_primary_window = -1;
  return m_instances.size() == 0;
}


////////////////////////
//Class Tag Functions//
////////////////////////

/**
*
* Sets tag to specified string
* @param: tag - specified string to set tag
*/
void Tag::SetTag(CString tag)
{
  m_tag = tag;
}

/**
*
* Returns the tag
* @returns: tag
*/
CString Tag::GetTag() const
{
  return m_tag;
}

/**
*
* Sets minchar to specified character
* @param: min - minimum character to be used in tag
*/
void Tag::Setminchar(TCHAR min)
{
  minchar = min;
}

/**
*
* Returns the minchar
* @returns: minimum character used in tag
*/
TCHAR Tag::Getminchar() const
{
  return minchar;
}

/**
*
* Sets maxchar to specified character
* @param: max - maximum character to be used in tag
*/
void Tag::Setmaxchar(TCHAR max)
{
  maxchar = max;
}

/**
*
* Returns the minchar
* @returns: maximum character used in tag
*/
TCHAR Tag::Getmaxchar() const
{
  return maxchar;
}

/**
*
* Postfix operator++
* @returns: increments tag
*/
Tag Tag::operator++(int num)
{
  int tagSize;
  TCHAR thisMaxchar;
  TCHAR thisMinchar;
  TCHAR tempchar;
  int asciiValue;
  
  thisMaxchar = Getmaxchar();  // get maxchar
  thisMinchar = Getminchar();  // get minchar
  
  tagSize = m_tag.GetLength();  // get length of tag
  
  if (tagSize != 0) // if tag is not empty
  {
    asciiValue = m_tag.GetAt(tagSize-1);
    
    if ('0' <= asciiValue && asciiValue <= '9') // if m_tag is type1 tag, NNNN
    {
      for (int i = tagSize - 1; i >= 0; i--) // loop through tag
      {
        asciiValue = m_tag.GetAt(i);
        if (asciiValue == '9') // if number equals nine, set to zero and continue loop
        {
          m_tag.SetAt(i, '0');
        }
        else  // else increment ascii value and break out of loop
        {
          asciiValue++;
          tempchar = asciiValue;
          m_tag.SetAt(i, tempchar);
          break;
        }
      }
    }
    else  // else m_tag is type2 tag, NNNL
    {
      if (m_tag.GetAt(tagSize-1) == thisMaxchar)  // if letter is equal to maxchar reset to minchar
      {
        m_tag.SetAt(tagSize-1, thisMinchar);
        
        for (int i = tagSize - 2; i >= 0; i--) // loop through tag
        {
          asciiValue = m_tag.GetAt(i);
          if (asciiValue == '9') // if number equals nine, set to zero and continue loop
          {
            m_tag.SetAt(i, '0');
          }
          else  // else increment ascii value and break out of loop
          {
            asciiValue++;
            tempchar = asciiValue;
            m_tag.SetAt(i, tempchar);
            break;
          }
        }
      }
      else  // else increment letter
      {
        asciiValue = m_tag.GetAt(tagSize-1) + 1;
        tempchar = asciiValue;
        m_tag.SetAt(tagSize-1, tempchar);
      }
    }
  }
  return * this;
}

/**
*
* Postfix operator--
* @returns: decrements tag
*/
Tag Tag::operator--(int num)
{
  int tagSize;
  TCHAR thisMaxchar;
  TCHAR thisMinchar;
  TCHAR tempchar;
  int asciiValue;
  
  thisMaxchar = Getmaxchar();  // get maxchar
  thisMinchar = Getminchar();  // get minchar
  
  tagSize = m_tag.GetLength();  // get length of tag
  
  if (tagSize != 0) // if tag is not empty
  {
    asciiValue = m_tag.GetAt(tagSize-1);
    
    if (48 <= asciiValue && asciiValue <= 57) // if m_tag is type1 tag, NNNN
    {
      for (int i = tagSize - 1; i >= 0; i--) // loop through tag
      {
        asciiValue = m_tag.GetAt(i);
        if (asciiValue == 48) // if number equals zero, set to nine and continue loop
        {
          m_tag.SetAt(i, '9');
        }
        else  // else decrement ascii value and break out of loop
        {
          asciiValue--;
          tempchar = asciiValue;
          m_tag.SetAt(i, tempchar);
          break;
        }
      }
    }
    else  // else m_tag is type2 tag, NNNL
    {
      if (m_tag.GetAt(tagSize-1) == thisMinchar)  // if letter is equal to minchar reset to maxchar
      {
        m_tag.SetAt(tagSize-1, thisMaxchar);
        
        for (int i = tagSize - 2; i >= 0; i--) // loop through tag
        {
          asciiValue = m_tag.GetAt(i);
          if (asciiValue == 48) // if number equals zero, set to nine and continue loop
          {
            m_tag.SetAt(i, '9');
          }
          else  // else decrement ascii value and break out of loop
          {
            asciiValue--;
            tempchar = asciiValue;
            m_tag.SetAt(i, tempchar);
            break;
          }
        }
      }
      else  // else decrement letter
      {
        asciiValue = m_tag.GetAt(tagSize-1) - 1;
        tempchar = asciiValue;
        m_tag.SetAt(tagSize-1, tempchar);
      }
    }
  }
  return * this;
}

////////////////////////
//Class Tube Functions//
////////////////////////

/**
*
* Default constructor
*/
Tube::Tube()
{     
  m_next_root_id = 1;
  //AfxMessageBox("Default Tube Constructor");
}

/**
*
* Constructor that sets the tag with Cstring tag argument. Each physical tube can have multiple transects.
* For simplicity the transects are just referenced as another tube.
* @param tag - the tag to name the tube
*/
Tube::Tube(const CString& tag)
{
  m_next_root_id = 1;
  m_tag = tag; 
  //AfxMessageBox("Tag Set Tube Constructor");
}


/**
*
* Default destructor
*/
Tube::~Tube()
{
  
}


/**
*
* Clears the vector of roots
* keeps tag the same.
* @returns: true on success, false on fail.
*/
bool Tube::ClearAll()
{
  m_root.clear();
  
  m_tag = "";
  return m_root.size() == 0;
}

/**
* MAY NEED TO DEPRECATE, SEE METHOD BELOW
* Creates a new root and adds it to the m_root vector
* @returns: address of newly created root
*/
Root* Tube::CreateRoot()
{  
  
  Root new_root( m_next_root_id );
  m_next_root_id++;
  
  m_root.push_back(new_root);
  return &m_root.back();
}


/**
*
* Creates a new root and adds it to the m_root vector
* @params: window - current window number to add root to
* @returns: address of newly created root
*/
Root* Tube::CreateRoot(int w)
{
  Root new_root( m_next_root_id++); 
  new_root.SetPrimaryWindow(w);
  new_root.SetWindowID(GetNextRootWindowID(w));
  m_root.push_back(new_root);
  return &m_root.back();  
}


/**
*
* Removes a root with specified position root_number from m_root
* @param root_number - the root position
* @returns: bool indicating success (0 good, 1 failure)
*/
bool Tube::DeleteRoot(int root_id)
{

  Root *returned_root;
  vector<Root>::iterator r_ptr;

  for( r_ptr = m_root.begin(); r_ptr != m_root.end(); r_ptr++ )
  {
    if( r_ptr->GetID() == root_id)
    {
      returned_root = m_root.erase(r_ptr);
      break;
    }
  }
  
  if(returned_root == NULL)
  {
    AfxMessageBox("Error Delete");
	  return false;
  }
  else
  {
//    AfxMessageBox("Root Deleted");
    return true;
  }
}

/**
*
* set tube with specified tag
* @param tag - the name of the tube
*/
void Tube::SetTubeTag(const CString& tag)
{
  m_tag = tag;
}

/**
*
* get tube tag
* @return CString
*/
CString Tube::GetTag() const
{
  return m_tag;
}

/**
* gets the root of specified ID
* @return address of root
*/
Root* Tube::GetRoot(int ID)
{
  if (ID<1 || ID>=m_next_root_id)
    return NULL;
  
  int i = 0;
  vector<Root>::iterator iter;
  for(iter = m_root.begin();iter!=m_root.end();iter++){
    if(iter->GetID()==ID)
      return &m_root.at(i);
    i++;
  }
  return NULL;
  
  //return &m_root.at(index-1);
}

const Root* Tube::GetRoot(int ID) const
{
    if (ID<1 || ID>=m_next_root_id)
    return NULL;
  
  int i = 0;
  vector<Root>::const_iterator iter;

  for(iter = m_root.begin();iter!=m_root.end();iter++){
    if(iter->GetID()==ID)
      return &m_root.at(i);
    i++;
  }
  return NULL;
}

/**
* Given a window number, this will fill a vector with root objects and return it to the user
* @param window The window number to search for roots within
* @return pointer to a vector of roots, NULL if window is invalid
*/

std::vector<Root>* Tube::GetRootsInWindow(int window)
{
  assert(window >= 1);

  //If the function received an invalid window number, return NULL
  if(window < 1)
    return NULL;

  //If the vector is not empty, clear it.
  if(m_roots_in_window.size() > 0)
    m_roots_in_window.clear();


  for(int i = 0; i < m_root.size(); i++)
  {
    if(m_root.at(i).GetPrimaryWindow() == window)
      m_roots_in_window.push_back(m_root.at(i));
  }

  return &m_roots_in_window;
}

int Tube::GetNextRootWindowID(int primary_window)
{
  assert(primary_window >= 1);
  int max_id = 0;
  if(primary_window < 1)    return -3;
  for( int i = 0; i < m_root.size(); i++ )
  {
    if( m_root.at(i).GetPrimaryWindow() == primary_window )
    {
      int id = m_root.at(i).GetWindowID();
      if (id > max_id)  max_id = id;
    }
  }
  return max_id + 1;
}

// returns whether or not imaged for the
// current tube window and session has been scanned by the automatic detector
const bool Tube::GetImageScanned(int window, int session) const
{
  for(int i = 0; i < m_image_scanned.size(); i++)
  {
    if(m_image_scanned.at(i).window == window && m_image_scanned.at(i).session == session)
    {
      return true;
    }
  }
  return false;
}

// sets image refrenced by window and session is scanned
// or removes the label depending on status
void Tube::SetImageScanned(bool status, int window, int session)
{
  vector<ImageScanned>::iterator v1;
  if(status)
  {
    // size isn't zero, check to see if already labeled 
    if(m_image_scanned.size() != 0)
    {
      v1 = m_image_scanned.begin();
      while(v1 != &m_image_scanned.back())
      {
        // already labeled as scanned, do nothing
        if((v1->window == window) && v1->session == session) return;
        v1++;
      }      
    }
    // not in vector, place at end
    ImageScanned temp(window,session);     
    m_image_scanned.push_back(temp);
  } else // remove an instace of a scanned image
  {
    // size isn't zero, check to see if already labeled 
    if(m_image_scanned.size() != 0)
    {
      v1 = m_image_scanned.begin();
      while(v1 != &m_image_scanned.back())
      {
        // already labeled as scanned, remove from list
        if((v1->window == window) && v1->session == session)
        {
          m_image_scanned.erase(v1);
          return;
        }
        v1++;
      }      
    }
  }
}


/**
*
* A constructor
*/
Experiment::Experiment()
{
  // By default, the experiment mm conversion is 1.0
  Experiment::SetMMConversion(1.0);

  // By default the experiment is not an old file convert
  m_is_old = false;
  

  //AfxMessageBox("Constructed new experiment");

}

/**
*
* A deconstructor
*/
Experiment::~Experiment()
{
  
}

/**
*
* @returns m_tubes - number of tubes in experiment
*/
int Experiment::GetNTubes() const
{
  return m_tubes.size();
}

/**
*
* Searches vector for matching tag. Because the tag could potentially return NULL, a pointer is used
* instead of a reference.
* @returns: address of tube
*/
Tube* Experiment::GetTube(const CString& tag)
{
  CString str;
  for (size_t ix=0 ; ix < m_tubes.size(); ++ix)    
  {                                             
    if (m_tubes[ix].GetTag() == tag)            
    {
      return &m_tubes.at(ix);
    }
  }
  //AfxMessageBox("Returning null tube");
  return NULL;  // changed by CBD 4/3/07
}

/**
*
* Searches vector for matching tag. Because the tag could potentially return NULL, a pointer is used
* instead of a reference.
* @returns: address of tube
*/
const Tube* Experiment::GetTube(const CString& tag) const
{                                              
  for (size_t ix=0 ; ix < m_tubes.size(); ++ix)
  {
    if (m_tubes[ix].GetTag() == tag)
    {
      return &m_tubes.at(ix);
    }
  }
  
  return NULL;
}

/**
*
* Clears the vector of Tubes, m_tubes
* keeps m_image_format, m_image_dir, and m_mm_conversion the same.
* @returns: true on success, false on fail.
*/
bool Experiment::ClearAll()
{
  m_tubes.clear();
  //should also clear the member vars

  // !!! Note:  The comment above says that we should not be changing 
  //  m_image_format, m_image_dir, or m_mm_conversion, but here you
  //  can see clearly that we are doing so.  My suggestion:
  //    either keep this code (and also clear the m_image_format), and change the comment, or
  //    modify this code and change the name of the function to ClearAlmostAll() or something
  //    like that (and test thoroughly). -- STB
  m_mm_conversion = 1;
  m_image_dir = "";

  // Set current experiment as NOT an old converted file
  m_is_old = false;
  
  return m_tubes.size() == 0;
}

/**
*
* Creates a tube with the specified tag
* @param tag - the name of the tube
* @returns: address of newly created tube
*/
Tube* Experiment::CreateTube(const CString& tag)
{
  Tube new_tube(tag);
  //AfxMessageBox("Tag: "+tag);
  m_app_data->SetTube(tag);  //added by jtm 4/4/07
  m_tubes.push_back(new_tube);
  return &m_tubes.back();
}

/**
*
* Removes tube with specified tag in the m_tubes vector
* @param tag - the name of the tube
* @returns: bool indicating success
*/
bool Experiment::DeleteTube(const CString& tag)
{
  
  for (size_t ix=0 ; ix < m_tubes.size(); ++ix)
  {
    if (m_tubes[ix].GetTag() == tag)
    {
      m_tubes.erase(m_tubes.begin()+ix);
      return true;
    }
  }
  
  return true;//false;
}



// this should be the authoritative source for the version information
const CString Settings::m_version = "Rootfly_20";

const COLORREF Settings::m_default_root_drawing_color = RGB(255, 0, 0);
const COLORREF Settings::m_default_root_drawing_color_selected = RGB(0, 255, 0);
const COLORREF Settings::m_default_root_drawing_color_dead = RGB(0, 0, 0);
const COLORREF Settings::m_default_root_drawing_color_dead_selected = RGB(200, 200, 200);

Settings::Settings()
{
  // initialize root drawing colors (unselected = red, selected = green, dead = black)
//  m_root_drawing_color = RGB(255, 0, 0);
//  m_root_drawing_color_selected = RGB(0, 255, 0);
//  m_root_drawing_color_dead = RGB(0, 0, 0);
  m_root_drawing_color = GetDefaultRootDrawingColor();
  m_root_drawing_color_selected = GetDefaultRootDrawingColorSelected();
  m_root_drawing_color_dead = GetDefaultRootDrawingColorDead();
  m_root_drawing_color_dead_selected = GetDefaultRootDrawingColorDeadSelected();
  m_condition_default_index = 0;
}

Settings::~Settings()
{
}

void Settings::SaveUserPreferences()
{
  CString condition_name;
  int condition_size = m_condition_vector.size();
  Registry reg_instance("Settings");
  
  reg_instance.SaveStringToReg("Version", m_version);
  reg_instance.SaveStringToReg("FileDir", m_file_dir);
  reg_instance.SaveIntToReg("ConditionCodeDefaultIndex", m_condition_default_index);
  
  reg_instance.SaveIntToReg("RootIdEnabled",RootIdIsEnabled());
  reg_instance.SaveIntToReg("DeadRootsEnabled",DeadRootsIsEnabled());
  reg_instance.SaveIntToReg("ShowDiametersEnabled",ShowDiametersIsEnabled());
  
  reg_instance.SaveIntToReg("RootDrawingColor", GetRootDrawingColor());
  reg_instance.SaveIntToReg("RootDrawingColorSelected", GetRootDrawingColorSelected());
  reg_instance.SaveIntToReg("RootDrawingColorDead", GetRootDrawingColorDead());
  reg_instance.SaveIntToReg("RootDrawingColorDeadSelected", GetRootDrawingColorDeadSelected());

  reg_instance.SaveIntToReg("ConditionSize", condition_size);
  for(int i = 0; i < condition_size; i++)
  {
    condition_name.Format("Condition_%d",i);
    reg_instance.SaveStringToReg(condition_name, m_condition_vector.at(i));
  }
  
  
  // TODO: save window position using GetWindowPlacement
}

// Sets 'color' to the color found in registry with key 'str'.
// If key not found, then does not set 'color'.
void GetColorFromReg(const Registry& reg, const CString& str, COLORREF* color)
{
  COLORREF col;
  col = reg.GetIntFromReg(str);
  if (col != -1)  *color = col;
}

void Settings::ReadUserPreferences()
{
  CString condition_name;
  const Registry reg_instance("Settings");
  m_file_dir = reg_instance.GetStringFromReg("FileDir");
  m_condition_default_index = reg_instance.GetIntFromReg("ConditionCodeDefaultIndex");
  if (m_condition_default_index < 0)  m_condition_default_index = 0;
  int condition_size = reg_instance.GetIntFromReg("ConditionSize");

  // all member bools below are set to true if registry key doesn't exist
  // making the default for all the checkboxes to be turned on
  m_show_root_ids = reg_instance.GetIntFromReg("RootIdEnabled") != 0 ? true:false;
  m_show_dead_roots = reg_instance.GetIntFromReg("DeadRootsEnabled") != 0 ?true:false;
  m_show_diameters = reg_instance.GetIntFromReg("ShowDiametersEnabled") != 0 ?true:false;
  
  GetColorFromReg(reg_instance, "RootDrawingColor", &m_root_drawing_color);
  GetColorFromReg(reg_instance, "RootDrawingColorSelected", &m_root_drawing_color_selected);
  GetColorFromReg(reg_instance, "RootDrawingColorDead", &m_root_drawing_color_dead);
  GetColorFromReg(reg_instance, "RootDrawingColorDeadSelected", &m_root_drawing_color_dead_selected);

  m_condition_vector.clear();
  for(int i = 0; i < condition_size; i++)
  {
    condition_name.Format("Condition_%d",i);
    CString code = reg_instance.GetStringFromReg(condition_name);
    
    // Check to make sure that code is valid.  This should never be a problem unless
    // the user manually edits the registry, because ManageConditionCodesDlg::AddNewCode() 
    // checks for validity whenever the user enters a new code.
    // Nevertheless, this is still worth checking because an invalid 
    // condition code will prevent .rfy files from being loaded properly (fgets
    // skips over leading whitespace), which
    // can cause severe data loss for the user.  Better not to risk it.
    if (code.GetLength() > 0 && code[0] != ' ' && code[0] != '\t')
    {
      m_condition_vector.push_back(code);
    }
  }
  
}

ApplicationData::ApplicationData()
{
  m_modified = false;
}
/*

  bool ApplicationData::ClearAll()
  {
  bool m_modified = false;    
  m_filename = "";
  m_curr = CurrentImageProperties();
  m_windowImgBuffer = ImageBuffer() ;   
  m_sessionImgBuffer = ImageBuffer() ; 
  return true;
  }
*/
ApplicationData::~ApplicationData()
{
  
}

// completely reloads the image buffers,
void ApplicationData::ReloadImageBuffers(bool reload_window, bool reload_session, CString dir, ImageFileFormat file_format)
{
  assert(NTHUMBNAILS == 5);  // if this is not true, then we will have to rewrite the hardcoded 2s and 3s below (which really should be NTHUMBNAILS/2)

  CString name = "";
  ImageInfo ii;

  if(reload_session)
  {
    // reloads the whole session buffer along with the dates
    for(int i = -2; i < 3; i++)
    {
      name = file_format.GetFileName(dir,m_curr.m_tube,m_curr.m_window,m_curr.m_session+i);
      assert(i+2>=0 && i+2<NTHUMBNAILS);  
      if(name != "") {
        ii = file_format.GetImageInfo(name);
        bool success = blepo::Load(name, &m_session_buffer[i+2]);
        assert(success);
        m_app_session_dates[i+2].Format("%d/%d/%d",ii.date.month,ii.date.day,ii.date.year);                 
      } else
      {
        m_app_session_dates[i+2] = "";
      }
      name = "";
    }
  }
  name = "";
  if(reload_window)
  {
    // reloads the whole window buffer
    for(int i = -2; i < 3; i++)
    {
      name = file_format.GetFileName(dir,m_curr.m_tube,m_curr.m_window+i,m_curr.m_session);
      if(name != "")
      {        
        blepo::Load(name, &m_window_buffer[i+2]);
        name = "";
      }
    }    
  }
}


// shifts the window buffer and loads up images needed into the buffer
void ApplicationData::UpdateWindowBuffer(int changed, CString dir, ImageFileFormat file_format)
{
  blepo::ImgBgr temp[NTHUMBNAILS];
  CString name = "";
  ImageInfo ii;
  
  // reloads the whole session buffer and window buffer if it has been changed by more
  // than five
  if((changed >= NTHUMBNAILS) || (changed <= -NTHUMBNAILS))
  {
    ReloadImageBuffers(true, true, dir, file_format);
    return;
  }else
  {
    ReloadImageBuffers(false, true, dir, file_format);
  }
  
  // copies the whole buffer into a temp buffer
  for(int i = 0; i < NTHUMBNAILS; i++)
  {
    temp[i] = m_window_buffer[i];
  }
  
  if(changed> 0)
  {
    // copies the temp images that will remain in the buffer
    // after the shift. repopulates only ones needed if window was changed less than 5
    for(i = changed; i < NTHUMBNAILS; i++)
    {
      m_window_buffer[i-changed] = temp[i];
    }
    for(i = NTHUMBNAILS - changed; i < NTHUMBNAILS; i++)
    {
      name = file_format.GetFileName(dir,m_curr.m_tube,m_curr.m_window+i-2,m_curr.m_session);
      assert(i>=0 && i<NTHUMBNAILS);  
      if(name != "")
      {
        ii = file_format.GetImageInfo(name);
        blepo::Load(name, &m_window_buffer[i]);
      }
      name = "";
    }
  } else
  {
    // copies the temp images that will remain in the buffer
    // after the shift. repopulates only ones needed if window was changed less than 5
    for(i = 0; i < NTHUMBNAILS+changed; i++)
    {
      assert(i>=0 && i<NTHUMBNAILS);  
      assert(i-changed>=0 && i-changed<NTHUMBNAILS);  
      m_window_buffer[i-changed] = temp[i];
    }
    for(i = 0; i < -1*changed; i++)
    {
      name = file_format.GetFileName(dir,m_curr.m_tube,m_curr.m_window+i-2,m_curr.m_session);
      assert(i>=0 && i<NTHUMBNAILS);  
      if(name != "")
      {
        ii = file_format.GetImageInfo(name);
        blepo::Load(name, &m_window_buffer[i]);
      }
      name = "";
    }
  }
}

// shifts the session buffer and loads up images needed into the buffer
void ApplicationData::UpdateSessionBuffer(int changed, CString dir, ImageFileFormat file_format)
{
  blepo::ImgBgr temp[NTHUMBNAILS];
  CString temp_dates[NTHUMBNAILS];
  CString name = "";
  ImageInfo ii;
  
  // reloads the whole window buffer and session buffer if it has been changed by more
  // than five
  if((changed >= NTHUMBNAILS) || (changed <= -NTHUMBNAILS))
  {
    ReloadImageBuffers(true, true, dir, file_format);
    return;
  }else
  {
    ReloadImageBuffers(true, false, dir, file_format);
  }
  
  // copies the whole buffer into a temp buffer
  for(int i = 0; i < NTHUMBNAILS; i++)
  {
    assert(i>=0 && i<NTHUMBNAILS);
    temp[i] = m_session_buffer[i];
    temp_dates[i] = m_app_session_dates[i];
  }
  
  if(changed> 0)
  {
    // copies the temp images that will remain in the buffer
    // after the shift. repopulates only ones needed.
    for(i = changed; i < NTHUMBNAILS; i++)
    {
      assert(i-changed>=0 && i-changed<NTHUMBNAILS);  
      assert(i>=0 && i<NTHUMBNAILS);
      m_session_buffer[i-changed] = temp[i];
      m_app_session_dates[i-changed] = temp_dates[i];
    }
    for(i = NTHUMBNAILS - changed; i < NTHUMBNAILS; i++)
    {
      name = file_format.GetFileName(dir,m_curr.m_tube,m_curr.m_window,m_curr.m_session+i-2);
      assert(i>=0 && i<NTHUMBNAILS);
      if(name != "") 
      {
        ii = file_format.GetImageInfo(name);
        blepo::Load(name, &m_session_buffer[i]);
        m_app_session_dates[i].Format("%d/%d/%d",ii.date.month,ii.date.day,ii.date.year);                 
      }else 
      {
        m_app_session_dates[i] = "";
      }
      name = "";
    }
  } else
  {
    // copies the temp images that will remain in the buffer
    // after the shift. repopulates only ones needed
    for(i = 0; i < NTHUMBNAILS+changed; i++)
    {
      assert(i-changed>=0 && i-changed<NTHUMBNAILS);  
      assert(i>=0 && i<NTHUMBNAILS);  
      m_session_buffer[i-changed] = temp[i];
      m_app_session_dates[i-changed] = temp_dates[i];
    }
    for(i = 0; i < -1*changed; i++)
    {
      name = file_format.GetFileName(dir,m_curr.m_tube,m_curr.m_window,m_curr.m_session+i-2);
      assert(i>=0 && i<NTHUMBNAILS);  
      if(name != "")
      {
        ii = file_format.GetImageInfo(name);
        blepo::Load(name, &m_session_buffer[i]);
        m_app_session_dates[i].Format("%d/%d/%d",ii.date.month,ii.date.day,ii.date.year);                 
      } else
      {
        m_app_session_dates[i] = "";
      }
      name = "";
    }
  }
}

bool Experiment::SetImageFormat( CString format )
{
  // if( format == ( jpeg || bmp ) )...etc, may implement this later
  m_image_format = format;
  return m_image_format?true:false;  
}

CString Experiment::GetImageFormat() const
{
  return m_image_format;       
}

bool Experiment::SetImageDir( CString directory )
{
  m_image_dir = directory;     
  return m_image_dir?true:false;     // this may check for valid dir later
}

const CString Experiment::GetImageDir() const
{
  return m_image_dir;          
}

bool Experiment::SetMMConversion( float mm_conversion )
{
  m_mm_conversion = mm_conversion;   
  return m_mm_conversion?true:false; 
}

float Experiment::GetMMConversion() const
{
  return m_mm_conversion;      
}


/**
* Default Constructor
*/
RootflyData::RootflyData() 
{
  m_exp.SetAppData(m_appdata);
}

/**
* Default Destructor
*/
RootflyData::~RootflyData()
{
}



// registry methods probably need error checking...
void Registry::SaveStringToReg(const CString& name, const CString& data)
{
  pApp->WriteProfileString(m_section, name, data);
}

CString Registry::GetStringFromReg(const CString& name) const
{
  // if entry not found in registry, return null string
  return pApp->GetProfileString(m_section, name, "");
}

void Registry::SaveIntToReg(const CString& name, const int& data)
{
  pApp->WriteProfileInt(m_section, name, data);
}

// if entry not found in registry, returns -1
int Registry::GetIntFromReg(const CString& name) const
{
  return pApp->GetProfileInt(m_section, name, -1);
}





///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// ///////////////////////////////////////////////////////////////////////// //
// /////////////                                            //////////////// //
// /////////////            FILE I/O Group Code             //////////////// //
// /////////////                                            //////////////// //
// ///////////////////////////////////////////////////////////////////////// //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////





/**
* Loads a RootInstance of RPoints and RDiameters into the current data structure.
*
* The file pointer will be located at the beginning of a RootInstance
* and the function will load all points and diameters associated with the 
* RootInstance
*
* @author Hamilton Evans
* @param fpt pointer to an open file handle.
* @return Success (0) or Failure (-1)
*/
int RootInstance::Load(FILE *fpt)
{
  char          tempBuf[1000];
  int           i, num_points, num_diameters, color, x, y, window, session_num, liveordead;
  double        radius = 0.0;
  CString       user_code;
  RPoint        *new_point;
  RDiameter     *new_diameter;


  fscanf(fpt,"%d %d %d %d %d\n", &session_num, &num_points, &num_diameters, &color, &liveordead); 

  // Note:  If the condition code is the empty string, then this code will fail.
  // It will ignore the empty string and instead treat the next line as the condition code.
  // It also eats up leading whitespace in a condition code.
  // Our solution for now is not to allow empty string condition codes or codes with
  // leading whitespace.
  char *status;
  status = fgets ( tempBuf, 1000, fpt);
  tempBuf[strlen(tempBuf)-1] = '\0';
  m_condition_code = tempBuf;
  

  // checks if we were able to read the file
  if(status == NULL)
  {
    fclose(fpt);
    return -1;
  }
  
    
  SetColor((enum RootInstance::Color)color);
  
  m_is_alive = (liveordead == 1);
  
  SetSessionNumber(session_num);
  
  // Load the points for the instance
  for(i=0;i<num_points;i++)
  {
    new_point = (RPoint*)CreatePoint();
    
    fscanf(fpt,"%d %d %d\n",&x,&y,&window); 
    new_point->x = x;
    new_point->y = y;
    new_point->window = window;
  }
  
  // Load the diameters for the instance
  for(i=0;i<num_diameters;i++)
  {
    new_diameter = (RDiameter*)CreateDiameter();
    
    fscanf(fpt,"%d %d %d %lf\n",&x,&y,&window,&radius);
    new_diameter->radius = radius;
    new_diameter->r_point.x = x;
    new_diameter->r_point.y = y;
    new_diameter->r_point.window = window;
  }
  
  return 0;
}

/**
* Saves information from the current datastructure
* @param fp pointer to an open file handle.
* @return Success (0) or Failure (-1)
*/
int RootInstance::Save(FILE *fp) const
{
  g_debug_logfile.Print("RootInstance::Save() 1\n");
  int num_points, num_diams, color, status, i;
  
  num_points  = m_points.size();
  num_diams   = m_diameters.size();
  color       = m_color;
  status      = (m_is_alive) ? 1 : 0;
  
  g_debug_logfile.Print("RootInstance::Save() 2\n");
  //Note, instead of the date this now saves the session number as an int only.
  fprintf(fp, "%d %d %d %d %d\n%s\n", m_session_number, num_points, num_diams, color, status, m_condition_code);
  
  //Put the points in the file
  g_debug_logfile.Print("RootInstance::Save() 3 %d <= %d\n", num_points, m_points.size());
  assert(num_points == m_points.size());
  for(i=0; i<num_points; i++) 
  {
    if(fprintf(fp, "%d %d %d\n",m_points.at(i).x, m_points.at(i).y, m_points.at(i).window) < 0)
    {
      return -1;
    }
    
  }
  g_debug_logfile.Print("RootInstance::Save() 4 %d <= %d\n", num_diams, m_diameters.size());
  assert(num_diams == m_diameters.size());
  //Put the diameters in the file
  for(i=0; i<num_diams; i++) {
g_debug_logfile.Print("RootInstance::Save() 4b %d\n", i);
    if(fprintf(fp, "%d %d %d %f\n",   m_diameters.at(i).r_point.x, 
      m_diameters.at(i).r_point.y, 
      m_diameters.at(i).r_point.window, 
      m_diameters.at(i).radius) < 0)
    {
      return -1;
    }
    
  }
  g_debug_logfile.Print("RootInstance::Save() 5\n");
  
  
  return 0;
}


/**
* Loads a Root of RootInstances into the current data structure.
*
* The file pointer will be located at the beginning of a Root and the 
* function will load all RootInstances associated with the Root.
*
* @author Hamilton Evans
* @param fpt pointer to an open file handle.
* @return Success (0) or Failure (-1)
*/
int Root::Load(FILE *fpt)
{
  int           i, first_session, num_sessions, root_id, window_id, primary_window;
  RootInstance  *new_instance;
  
  fscanf(fpt,"\n%d %d %d %d %d\n",&first_session,&num_sessions, &root_id, &window_id, &primary_window);
  
  m_id = root_id;
  m_primary_window = primary_window;
  m_window_id = window_id;

  /// Begin numbering sessions at first_session, end when the number
  /// of sessions for a specific root are completed (i.e. - num_sessions).
  for(i=0;i<num_sessions;i++)
  {
    new_instance = (RootInstance*)CreateRootInstance(first_session+i);
    new_instance->Load(fpt);
  }
  
  return 0;
}

/**
* Saves information from the current datastructure
* @param fp pointer to an open file handle.
* @return Success (0) or Failure (-1)
*/
int Root::Save(FILE *fp) const
{
  int i, num_sessions=0, first_session=0;
  bool run=false;

  num_sessions  = m_instances.size();
  
  //If there are any instances defined, grab the session number of the first root instance.
  if(num_sessions > 0) {
    first_session = m_instances.at(0).GetSessionNumber();
  }
  
  if(fprintf(fp, "%d %d %d %d %d\n", first_session, num_sessions, m_id, m_window_id, m_primary_window) < 0) 
  {
    return -1;
  }
  
  
  for(i=0; i<num_sessions; i++) 
  {
    run=true;
    if((m_instances.at(i)).Save(fp) == -1) 
    {
      return -1;
    }
    fprintf(fp, "\n");
  }

  if(!run) 
  {
    fprintf(fp,"\n");
  }
  
  return 0;
}


/**
* Loads a Tube of Roots into the current data structure.
*
* The file pointer will be located at the beginning of a Tube and the 
* function will load all Roots associated with the Tube.
*
* @author Hamilton Evans
* @param fpt pointer to an open file handle.
* @return Success (0) or Failure (-1)
*/
int Tube::Load(FILE *fpt)
{
  int           i, num_roots, size, temp_window, temp_session;
  Root          *new_root;
  ImageScanned temp;
  
  // loads size of imagescanned vector
  fscanf(fpt,"%d\n", &size);
  // save imagescanned vector
  for(i = 0; i < size; i++)
  {
    
    fscanf(fpt, "%d %d\n", &temp_window, &temp_session);
    temp.window = temp_window;
    temp.session = temp_session;
    m_image_scanned.push_back(temp);
  }

  /// Read the file to get the number of roots and the tag for the tube.
  fscanf(fpt,"%d\n", &num_roots);                           
  
  for(i=0;i<num_roots;i++)
  {
    new_root = (Root*) CreateRoot();
    new_root->Load(fpt);
  }
  
  { // set 'm_next_root_id'
    int max_root_id = -1;
    for (i=0 ; i<m_root.size() ; i++)
    {
      int id = m_root[i].GetID();
      if (id >= max_root_id)  max_root_id = id;
    }
    m_next_root_id = max_root_id + 1;
  }

  return 0;
}

/**
* Saves information from the current datastructure
* @param fp pointer to an open file handle.
* @return Success (0) or Failure (-1)
*/
int Tube::Save(FILE *fp) const
{
  g_debug_logfile.Print("Tube::Save() 1\n");
  int num_roots, i;
  
  num_roots = m_root.size();
  g_debug_logfile.Print("Tube::Save() 2 %d\n", num_roots);
  
  
  if(fprintf(fp, "%s\n", m_tag) < 0) 
  {
    return -1;
  }
  g_debug_logfile.Print("Tube::Save() 3\n");

  // save imagescanned size
  if(fprintf(fp, "%d\n", m_image_scanned.size()) < 0)
  {
    return -1;
  }
  g_debug_logfile.Print("Tube::Save() 4\n");
  // save imagescanned vector
  for(i = 0; i < m_image_scanned.size(); i++)
  {
  g_debug_logfile.Print("Tube::Save() 4b %d\n", i);
    if(fprintf(fp, "%d %d\n", m_image_scanned.at(i).window,
                           m_image_scanned.at(i).session) < 0)
    {
      return -1; // problems with saving
    }
    
  }
  
  g_debug_logfile.Print("Tube::Save() 5\n");
  if(fprintf(fp, "%d\n", num_roots) < 0) 
  {
    return -1;
  }
  
  
  g_debug_logfile.Print("Tube::Save() 6 %d <= %d\n", num_roots, m_root.size());
  assert(num_roots == m_root.size());
  for(i=0; i<num_roots; i++) {
  g_debug_logfile.Print("Tube::Save() 6b %d\n", i);
    if((m_root.at(i)).Save(fp) == -1) 
    {
      return -1;
    }
  }
  g_debug_logfile.Print("Tube::Save() 7\n");
  return 0;
}

/**
* Loads a Experiment of Tubes into the current data structure.
*
* The file pointer will be located at the beginning of a Tube and the 
* function will load all Roots associated with the Tube.
*
* @author Hamilton Evans
* @param *filename the filename to be opened and loaded.
* @param *parameters a pointer to a Settings object.
* @return Success (0) or Failure (one of the nonzero ERRORS)
*/
int Experiment::Load(char *filename, Settings *parameters)
{  
  int 			    num_tubes;	
  char          version[20], tempBuf[1000];
  float 		    conversion;
  CString       image_dir = " ", tube_tag = " ";
  FILE			    *fpt;
  Tube			    *new_tube; 
  
  fpt = fopen(filename, "r");
  
  /// If the file cannot be opened, the function will return -1.
  if(fpt == NULL) 
  { 																	
    return ERRORS::RFY_LOADERROR_NOFILEFOUND;
  }

  // check to make sure that a valid rootfly file is being opened
  // first line must read Rootfly_XX although only up to the _ is checked
  CString file_version = fgets(version, 9, fpt);
  if (file_version != "Rootfly_")
  {
    fclose(fpt);
    return ERRORS::RFY_LOADERROR_FILECORRUPT;
  }

  //Clear Experiment
  if(!Experiment::ClearAll())
  {
    fclose(fpt);
    return ERRORS::RFY_LOADERROR_FILECORRUPT;
  }

  // Get the version number in the file
  fscanf(fpt,"%20s\n",&version);  

  // If the file is an old Rootfly file ("Rootfly_11"), use Experiment::LoadOld()
  if(strcmp(version,"11") == 0)
  {
    fclose(fpt);
    DWORD ret = AfxMessageBox("File was created with an old version of Rootfly.  Convert to the new format?", MB_YESNO);
    if (ret != IDYES)  
    {
      return ERRORS::RFY_LOADERROR_USERCANCELED;
    }
    // Set old file to true
    m_is_old = true;
    int status = LoadOld(filename, parameters);
    return status;
  }
  
  // If the file is a new Rootfly file ("Rootfly_20"), use Experiment::LoadOld()
  else if ( (file_version + version) == parameters->GetVersion())
  {
    // Current experiment is NOT an old converted file
    SetOldFile(false);

    //Read the image directory with spaces
    char *status;
    status = fgets ( tempBuf, 1000, fpt);
    tempBuf[strlen(tempBuf)-1] = '\0';
    image_dir = tempBuf;
    
    if(status == NULL)
    {
      fclose(fpt);
      return ERRORS::RFY_LOADERROR_FILECORRUPT;
    }
 
    status = m_image_file_format.Load(fpt);

    if(status == NULL)
    {
      fclose(fpt);
      return ERRORS::RFY_LOADERROR_FILECORRUPT;
    }

    // loads session dates
    int max_sessions;
    int temp_int;
    Date temp_session;
    CString message;
    fscanf(fpt,"%d\n",&max_sessions);
    for(int i = 1; i <= max_sessions; i++)
    {
      fscanf(fpt,"%d %d/%d/%d\n", &temp_int, &temp_session.month,
        &temp_session.day, &temp_session.year);
      m_session_dates.push_back(temp_session);
    }
    
    if(fscanf(fpt,"%f\n%d\n\n",&conversion,&num_tubes) == EOF)
    {
      fclose(fpt);
      return ERRORS::RFY_LOADERROR_FILECORRUPT;
    } 
    
    // Set internal variables
    m_image_dir = image_dir;
    m_mm_conversion = conversion;
    
    
    // Load all Tubes, this will recursively load entire data structure.
    for(i=0;i<num_tubes;i++)
    {
      status = fgets ( tempBuf, 1000, fpt);
      tempBuf[strlen(tempBuf)-1] = '\0';
      tube_tag = tempBuf;
      
      if(status == NULL)
      {
        fclose(fpt);
        return ERRORS::RFY_LOADERROR_FILECORRUPT;
      }
      
      new_tube = (Tube*) CreateTube( tube_tag );        
      new_tube->Load(fpt);     
    }
  }
  
  else
  {
    // unrecognized rootfly file version number
    fclose(fpt);
    return ERRORS::RFY_LOADERROR_VERSION;
  }

  fclose(fpt);
  return ERRORS::RFY_LOADERROR_SUCCESS;
}

/**
* Saves information from the current datastructure.
* So far the settings are not visible to this object, this will need to change.
*
* @param fp pointer to an open file handle.
* @param *parameters pointer to a Settings object
* @return Success (0) or Failure (-1)
*/

int Experiment::Save(char *filename, Settings *parameters) const
{
  g_debug_logfile.Print("Experiment::Save() 1 %s %x\n", filename, parameters);
  FILE *fp = fopen(filename, "w");
  g_debug_logfile.Print("Experiment::Save() 2 %d\n", fp);
  CString version   = parameters->GetVersion();
  unsigned int num_tubes = m_tubes.size();
  unsigned int i;
  
  
  if(fp == NULL) 
  {
    return -1;
  }
  
  g_debug_logfile.Print("Experiment::Save() 3 %s %s\n", version, m_image_dir);
  if (fprintf(fp,"%s\n%s\n",version,m_image_dir) < 0) return -1;

  g_debug_logfile.Print("Experiment::Save() 4\n");
  m_image_file_format.Save(fp);
  g_debug_logfile.Print("Experiment::Save() 5\n");
  
  // write date to file
  g_debug_logfile.Print("Experiment::Save() 6 %d <= %d\n", m_max_sessions, m_session_dates.size());
  assert(m_max_sessions == m_session_dates.size());  // we know this is failing, but why, and is it important? -- StB 10/17/07
  fprintf(fp,"%d\n", m_session_dates.size());
  g_debug_logfile.Print("Experiment::Save() 7\n");
  for(i = 0; i < m_session_dates.size() ; i++)
  {
  g_debug_logfile.Print("Experiment::Save() 7b %d\n", i);
    fprintf(fp,"%d %d/%d/%d\n",i,m_session_dates.at(i).month,
      m_session_dates.at(i).day, m_session_dates.at(i).year);
  }
  g_debug_logfile.Print("Experiment::Save() 8\n");
  
  // Write image format string
  //stringstream ss;
  //ImageFileFormatInfo iffi = m_image_file_format.GetImageFileFormatInfo();

  //char image_format[1024];
  //sprintf(image_format,"%d %s %s",iffi.numElementsInFileName,iffi.filetype,iffi.delim);

  //ss << iffi.numElementsInFileName << " " << iffi.filetype << " " << iffi.delim;
  //OrderElement oe;

  //for(i = 0; i < 7; i++)
  //{
	//oe = iffi.InfoOrder[i];
	//ss << " " << oe.name << " " << oe.element << " " << oe.prefix << " " << oe.length << " " << oe.included;
	//sprintf(image_format,"%s %s %d %s %d %d",image_format,oe.name,oe.element,oe.prefix,oe.length,oe.included);
  //sprintf(image_format,"%s %d %s %d %d",image_format,oe.element,oe.prefix,oe.length,oe.included);
  //}
  //CString image_format = ss.str().c_str();

  //if(fprintf(fp,"%s\n%s\n%s\n%f\n%d\n\n",version, m_image_dir, image_format, m_mm_conversion, num_tubes) < 0)
  //{
  //  return -1;
  //}
  g_debug_logfile.Print("Experiment::Save() 9\n");
  if(fprintf(fp,"%f\n%d\n\n",m_mm_conversion,num_tubes) < 0) return -1;
  assert(num_tubes == m_tubes.size());
  g_debug_logfile.Print("Experiment::Save() 10 %f %d <= %d\n", m_mm_conversion, num_tubes, m_tubes.size());
  
  //Find the number of sessions in the data structures
  for(i=0; i<num_tubes; i++) 
  {
  g_debug_logfile.Print("Experiment::Save() 10b %d\n", i);
    if((m_tubes.at(i)).Save(fp) == -1) 
    {
      return -1;
    }
    fprintf(fp, "\n\n");
  }
  g_debug_logfile.Print("Experiment::Save() 11\n");
  fclose(fp);
  g_debug_logfile.Print("Experiment::Save() 12\n");
  return 0;
}


/**
*  This function will load the old file format into the
*  old data structure.  That data structure will then 
*  call a function to convert the old data structure into
*  the new structure.
*
* \param filename of file that contains the old file format
* \param settings of current rootfly experiment
* \return returns 0 on success
*/


int Experiment::LoadOld(char *filename, Settings *parameters)
{
  
  OldData::RootflyData oldDataStructure;
  
  bool success;
  
  m_image_file_format.SetToBartz();
  success = oldDataStructure.Load((CString)filename);
  CString msg = "";
  int status = 0;
  
  if (success == true)
  {
    status = ConvertDataStructure(oldDataStructure, parameters, (CString)filename);
    if (status != 0)  return status;
    
//    if(!TestingData::compareDataStructure(oldDataStructure,this))
//    {
//      
//      return -1;
//    }
    
  }
  else
  {
    return ERRORS::RFY_LOADERROR_FILECORRUPT;
  }
  
  
  return 0;
}


/**
*  This function will convert an old data structure into the new
*  structure.  This will return 0 on success
*
* \author Corey Dickerson
* \param old data structure that will be used for conversion
* \param settings of current experiment
* \return returns 0 on success, -1 on failure
*/

int Experiment::ConvertDataStructure(OldData::RootflyData oldStructure, Settings *settings, const CString &filename)
{  
  CString currentVersion = settings->GetVersion();  
  int ntubes,ntransects,nwindows,nsessions;
  int t,w,p,tr,g, rb, s;
  int rootNum = 1;
  int winID = 1;
  CString temp_str;
  OldData::RootList *old_current_rootlist;
  int num_points, num_rootblocks;
  OldData::RootInstance *old_current_rootinstance;
  CPoint offset;
  bool status;
  
  Tube *currentTube;
  Root *currentRoot;
  RootInstance *currentInstance;
  RDiameter *currentDiameter;  
  RPoint currentPoint;  
  RPoint *currentRootPoint;  
  std::vector<CPoint> old_offsets;

  // checks if the parameter settings is not null, so that when we set the version number, we know it is valid
  if(settings == NULL)
  {
    return ERRORS::RFY_LOADERROR_FILECORRUPT;
  }
  
  // The next set of if statements set the important settings information, and return -1 if they are set incorrectly.

  // sets the image format for the data structure
  if(!SetImageFormat(oldStructure.m_image_format))
  { 
    return ERRORS::RFY_LOADERROR_FILECORRUPT;
  }

  // sets the image directory for the current experiment
  if(!SetImageDir(oldStructure.m_image_directory))
  {
    return ERRORS::RFY_LOADERROR_FILECORRUPT;
  }

  // sets the mm conversion for the current experiment
  if(!SetMMConversion(oldStructure.m_mm_conversion))
  {
    return ERRORS::RFY_LOADERROR_FILECORRUPT;
  }
  
  // Get number of tubes, transect, windows, and sessions
  oldStructure.m_root.GetDimensions(&ntubes,&ntransects,&nwindows,&nsessions);

  // loop through tubes
  for( t = 0; t < ntubes; t++ ) 
  {
    rootNum = 1;
    winID = 1;
    // loop through transects
    for( tr = 0; tr < ntransects; tr++ ) 
    {
      // Create Tube with appropriate tag
      temp_str.Format("%03d",t+1);

      currentTube = CreateTube(temp_str);
      
      // loop through windows
      for( w = 0; w < nwindows; w++ ) 
      {   
        // Get a list of all roots in given tube, transect, and window
        old_current_rootlist = oldStructure.m_root.GetRootList(t,tr,w);
        
        // Resize vector to be size of number of sessions
        // so there ARE holes
        old_offsets.resize(nsessions);
        
        // for each offset in all session
        for( g = 0; g < nsessions; g++) 
        {
          // Get offset for this current session
          offset = old_current_rootlist->offset_list.at(g);
          
          //Put offset in new vector at corresponding location
          old_offsets.at(g) = offset;
          
        }

        // get number of root blocks("ROOT") in old structure
        num_rootblocks = old_current_rootlist->size(); 
        if( num_rootblocks > 0 ) 
        {          
          // loop through root blocks
          for( rb = 0; rb < num_rootblocks; rb++ ) 
          {
            //Create new Root that will have root instances
            currentRoot = currentTube->CreateRoot(w+1);
            
            // Set root ID for each ROOT
            currentRoot->SetID(rootNum++);
            currentRoot->SetPrimaryWindow(w+1);
            currentRoot->SetWindowID(currentTube->GetNextRootWindowID(w+1)-1);     
            
            
            for(s = old_current_rootlist->at(rb).GetFirstSession() - 1; s < old_current_rootlist->at(rb).GetNumInstances()+old_current_rootlist->at(rb).GetFirstSession()-1; s++)
            {
              if(s >= old_offsets.size() )
                break;
              
              old_current_rootinstance = old_current_rootlist->at(rb).GetRootInstance(s,&status);
              num_points = old_current_rootinstance->points.size();
              
              // Create New root Instance  
              currentInstance = currentRoot->CreateRootInstance(s+1);
              currentInstance->SetConditionCode("NO_CODE_GIVEN");

              int color;
              
              switch(old_current_rootinstance->color)
              {
                case 0: color = 2; break;
                case 1: color = 1; break;
                case 2: color = 3; break;
                case 3: color = 0; break;
              }

              currentInstance->SetColor(color);
              
              //Set status of root instance (Dead or Alive)
              if(status)
                currentInstance->SetAlive();
              else
                currentInstance->SetDead();
              
              // loop through points
              for( p = 0; p < num_points; p++ ) 
              {
                //Create new point
                currentRootPoint = currentInstance->CreatePoint();
                
                //Set values of the points and then add offset                
                currentRootPoint->x = (old_current_rootinstance->points.at(p).x + old_offsets.at(s).x);
                currentRootPoint->y = (old_current_rootinstance->points.at(p).y + old_offsets.at(s).y);
                currentRootPoint->window = w + 1;
              } // end p
              
              // Create a new diameter
              currentDiameter = currentInstance->CreateDiameter();
              
              // Set the radius of the diameter
              currentDiameter->radius = old_current_rootinstance->diameter/ 2;// * diameterFactor;
              
              // Set the location of the diameter
              currentDiameter->r_point.x = (old_current_rootinstance->diameter_center.x );
              currentDiameter->r_point.y = (old_current_rootinstance->diameter_center.y);
              currentDiameter->r_point.window = w + 1;
                         
            } // end s
          } // end r
        }
        else 
        {
        } // end else
      } // end w
    } // end tr
  } // end t
  return 0;
}

// Converts coordinates of root points and diameters from an old Rootfly file (version 1.0.3)
// to the new version (2.0).  Note:  The old format stored client coordinates, which was not
// guaranteed to have the same aspect ratio as the image.  The new format stores image coordinates
// instead.
//
// @param dir  Directory containing images
// @param image_file_format  Specifies the format of the image names in the directory
void Experiment::ConvertOldRoots(const CString& dir, ImageFileFormat& image_file_format)
{
  CString tube_tag, name;
  int window, last_window = -1;
  int session;
  vector<Tube> *tube;
  vector<Tube>::iterator t_iter;
  vector<Root> *root;
  vector<Root>::iterator r_iter;
  vector<RootInstance> *inst;
  vector<RootInstance>::iterator i_iter;
  const vector<RPoint> *point;  
  const RDiameter *diam;
  RPoint conv_point, conv_radius;
  blepo::ImgBgr img;

  // Create temporary converter from client to image coordinates corresponding to old version of Rootfly (version 1.0.3).
  // (SetWindow is not called because the window does not matter for our purposes.  
  //  The window is ignored by ClientToImage anyway).
  ImageClientConverter comverter;
  { 
    CRect old_screen(0,0,639,482);  // dimensions of image display in old version of Rootfly (version 1.0.3), which could not be resized
    CString name = image_file_format.GetFirstImage(dir);  // grab dimensions of one image, just in case we happen to have roots whose images do not exist (which should not happen)
    int w = 640, h = 480;  // default, just in case we cannot find an image at all
    if (name != "" && blepo::Load(name, &img))  { w = img.Width();  h = img.Height(); }

    comverter.SetClientRect( old_screen );
    comverter.SetImageSize( CSize(w, h) );
  }

  tube =  GetAllTubes();
  if (!tube) return;

  // for each tube ...
  for( t_iter = tube->begin(); t_iter != tube->end(); t_iter++ )
  {    
    tube_tag = t_iter->GetTag();
    root = t_iter->GetAllRoots();
    if (!root) continue;

    // for each root in tube ...
    for( r_iter = root->begin(); r_iter != root->end(); r_iter++ )
    {
      inst = r_iter->GetAllRootInstances();
      if (!inst) continue;

      // for each root instance ...
      for( i_iter = inst->begin(); i_iter != inst->end(); i_iter++ )
      {        
        session = i_iter->GetSessionNumber();
        point = i_iter->GetAllPoints();
        if (!point) continue;

        // for each point ...
        for (int pt = 0; pt < point->size(); pt++ )
        {
          // load image for this tube, window, session (if not already loaded)
          // tell converter the dimensions of this image
          window = point->at(pt).window;
          if( window != last_window)
          {
            // get name of file
            name = image_file_format.GetFileName(dir,tube_tag,window,session);            
            // if a file was found, then load image and set converter based on image size
            if (name != "" && blepo::Load(name,&img))  comverter.SetImageSize( CSize(img.Width(),img.Height()) );
          }
          
          // convert from client to image coordinates
          conv_point = comverter.ClientToImage( point->at(pt) );
          i_iter->MovePoint(pt, conv_point.x,conv_point.y); 
          last_window = window;          
        }        
        last_window = -1;          

        // convert diameter in the same way
        diam = i_iter->GetDiameter(0);
        window = diam->r_point.window;
        conv_point = comverter.ClientToImage( diam->r_point );
        conv_radius = comverter.ClientToImage( RPoint(diam->r_point.x+diam->radius, diam->r_point.y+diam->radius, -1) );
        i_iter->MoveDiameter(0, conv_point.x, conv_point.y);
        i_iter->ResizeDiameter(0, conv_radius.x - conv_point.x);
        last_window = window;
      }
    }
  }

  // convert pixels/mm conversion
  // (Averages x and y.  Ideally these would be the same, but only if the old
  //  format happened to preserve the aspect ratio of the image.)
  conv_point = comverter.ClientToImage( RPoint( m_mm_conversion, m_mm_conversion, -1 ) );
  m_mm_conversion = (conv_point.x + conv_point.y) / 2.0;
}

void Experiment::ConvertOldRoots(const CString& dir, ImageFileFormat& image_file_format, CWnd *pParentWnd)
{
  CString tube_tag, name;
  int window, last_window = -1;
  int session;
  vector<Tube> *tube;
  vector<Tube>::iterator t_iter;
  vector<Root> *root;
  vector<Root>::iterator r_iter;
  vector<RootInstance> *inst;
  vector<RootInstance>::iterator i_iter;
  const vector<RPoint> *point;  
  const RDiameter *diam;
  RPoint conv_point, conv_radius;
  blepo::ImgBgr img;

  // progress bar creation
  int lower, upper;
  CProgressCtrl progress_bar;
  CRect rect, rect2, rect3;
  CClientDC dc(pParentWnd);
  dc.SetBkColor(::GetSysColor(COLOR_3DFACE));
  pParentWnd->GetWindowRect(rect);
  pParentWnd->ScreenToClient(rect);
  rect2 = CRect(rect.left+(rect.Width()/2)-150, rect.top+(rect.Height()/2)-15, rect.left+(rect.Width()/2)+150, rect.top+(rect.Height()/2)+15);
  rect3 = CRect(rect2.left,rect2.top-20,rect2.right,rect2.bottom-20);
  dc.DrawText("Converting Old Roots To New Format...         ",rect3,0);
  progress_bar.Create(WS_CHILD | WS_VISIBLE | PBS_SMOOTH , rect2, pParentWnd,0x16);


  // Create temporary converter from client to image coordinates corresponding to old version of Rootfly (version 1.0.3).
  // (SetWindow is not called because the window does not matter for our purposes.  
  //  The window is ignored by ClientToImage anyway).
  ImageClientConverter comverter;
  { 
    CRect old_screen(0,0,639,482);  // dimensions of image display in old version of Rootfly (version 1.0.3), which could not be resized
    CString name = image_file_format.GetFirstImage(dir);  // grab dimensions of one image, just in case we happen to have roots whose images do not exist (which should not happen)
    int w = 640, h = 480;  // default, just in case we cannot find an image at all
    if (name != "" && blepo::Load(name, &img))  { w = img.Width();  h = img.Height(); }

    comverter.SetClientRect( old_screen );
    comverter.SetImageSize( CSize(w, h) );
  }

  tube = GetAllTubes();
  if (!tube) return;

  
  // more progress bar
  lower = 0;
  upper = tube->size();
  progress_bar.SetRange(lower,upper);
  progress_bar.SetStep(1);

  // for each tube ...
  for( t_iter = tube->begin(); t_iter != tube->end(); t_iter++ )
  {    
    // progress bar step
    progress_bar.StepIt();

    tube_tag = t_iter->GetTag();
    root = t_iter->GetAllRoots();
    if (!root) continue;

    // for each root in tube ...
    for( r_iter = root->begin(); r_iter != root->end(); r_iter++ )
    {
      inst = r_iter->GetAllRootInstances();
      if (!inst) continue;

      // for each root instance ...
      for( i_iter = inst->begin(); i_iter != inst->end(); i_iter++ )
      {        
        session = i_iter->GetSessionNumber();
        point = i_iter->GetAllPoints();
        if (!point) continue;

        // for each point ...
        for (int pt = 0; pt < point->size(); pt++ )
        {
          // load image for this tube, window, session (if not already loaded)
          // tell converter the dimensions of this image
          window = point->at(pt).window;
          if( window != last_window)
          {
            // get name of file
            name = image_file_format.GetFileName(dir,tube_tag,window,session);            
            // if a file was found, then load image and set converter based on image size
            if (name != "" && blepo::Load(name,&img))  comverter.SetImageSize( CSize(img.Width(),img.Height()) );
          }
          
          // convert from client to image coordinates
          conv_point = comverter.ClientToImage( point->at(pt) );
          i_iter->MovePoint(pt, conv_point.x,conv_point.y); 
          last_window = window;          
        }        
        last_window = -1;          

        // convert diameter in the same way
        diam = i_iter->GetDiameter(0);
        window = diam->r_point.window;
        conv_point = comverter.ClientToImage( diam->r_point );
        conv_radius = comverter.ClientToImage( RPoint(diam->r_point.x+diam->radius, diam->r_point.y+diam->radius, -1) );
        i_iter->MoveDiameter(0, conv_point.x, conv_point.y);
        i_iter->ResizeDiameter(0, conv_radius.x - conv_point.x);
        last_window = window;
      }
    }
  }

  // convert pixels/mm conversion
  // (Averages x and y.  Ideally these would be the same, but only if the old
  //  format happened to preserve the aspect ratio of the image.)
  conv_point = comverter.ClientToImage( RPoint( m_mm_conversion, m_mm_conversion, -1 ) );
  m_mm_conversion = (conv_point.x + conv_point.y) / 2.0;
}